/*****************************************************************************
 
 i2clcd.c - LCD over I2C library 
		Designed for HD44870 based LCDs with I2C expander PCF8574X
		on Atmels AVR MCUs
 
 Copyright (C) 2006 Nico Eichelmann and Thomas Eichelmann
               2014 clean up by Falk Brunner 
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.
 
 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
 You can contact the authors at info@computerheld.de
 
*****************************************************************************/

/*
	Version 0.11
	Requires I2C-Library from Peter Fleury http://jump.to/fleury
	
	See i2clcd.h for description and example.
*/

#include <avr/pgmspace.h>
#include <stdbool.h>
#include <stdint.h> 
#include "defines.h"
#include <util/delay.h>
#include "i2clcd.h"
#include "i2cmaster.h"

/*****************************************************************************
 i2clcd.h - LCD over I2C library 
Designed for HD44870 based LCDs with I2C expander PCF8574X
Copyright (C) 2006 Nico Eichelmann and Thomas Eichelmann
2014 clean up by Falk Brunner
Example assignment, working with 
 PCF8574	<->		LCD
 ----------------------------------------------
 P0		<->		RS
 P1		<->		RW
 P2		<->		E
 P3		<->		LIGHT
 P4		<->		DB4
 P5		<->		DB5
 P6		<->		DB6
 P7		<->		DB7 
 
//--Display-Configuration-Settings-----------------------------------------------------------------------------------
*/

#define lcd_max_zl 4


#define LCD_LINES			4	        /**< Enter the number of lines of your display here */
#define LCD_COLS			20	        /**< Enter the number of columns of your display here */
#define LCD_LINE_MODE       LCD_2LINE   /**< Enter line mode your display here */

#define LCD_LINE1			0x00	    /**< This should be 0x00 on all displays */
#define LCD_LINE2			0x40	    /**< Change this to the address for line 2 on your display */
#define LCD_LINE3			20	    /**< Change this to the address for line 3 on your display */
#define LCD_LINE4			0x54	    /**< Change this to the address for line 4 on your display */
/*@}*/

#if LCD_LINES > 4
    #error "#define LCD_LINES must be less or equal to 4"
#endif

#if LCD_COLS > 20
    #error "#define LCD_COLS must be less or equal to 20"
#endif


//-------------------------------------------------------------------------------------------------------------------

//--The-following-definitions-are-corresponding-to-the-PIN-Assignment-(see-above)------------------------------------


#define LCD_D4_PIN			4	/**< LCD-Pin D4 is connected to P4 on the PCF8574 */
#define LCD_D5_PIN			5	/**< LCD-Pin D5 is connected to P5 on the PCF8574 */
#define LCD_D6_PIN			6	/**< LCD-Pin D6 is connected to P6 on the PCF8574 */
#define LCD_D7_PIN			7	/**< LCD-Pin D7 is connected to P7 on the PCF8574 */
#define LCD_RS_PIN			0	/**< LCD-Pin RS is connected to P0 on the PCF8574 */
#define LCD_RW_PIN			1	/**< LCD-Pin RW is connected to P1 on the PCF8574 */
#define LCD_E_PIN			2	/**< LCD-Pin E is connected to P2 on the PCF8574 */
#define LCD_LIGHT_PIN		3	/**< LCD backlight is connected to P3 on the PCF8574, low active */
/*@}*/

//-------------------------------------------------------------------------------------------------------------------

#if 0
#define LCD_D4				(1 << LCD_D4_PIN)	/**< bit 4 in 2nd lower nibble */
#define LCD_D5				(1 << LCD_D5_PIN)	/**< bit 5 in 2nd lower nibble */
#define LCD_D6				(1 << LCD_D6_PIN)	/**< bit 6 in 2nd lower nibble */
#define LCD_D7				(1 << LCD_D7_PIN)	/**< bit 7 in 2nd lower nibble */

#define LCD_RS				(1 << LCD_RS_PIN)	/**< RS-bit in 1st and 2nd higher nibble */
#define LCD_RW				(1 << LCD_RW_PIN)	/**< RW-bit in 1st and 2nd higher nibble */
#define LCD_LIGHT_N			(1 << LCD_LIGHT_PIN)/**< LCD backlight control, low active */
#define LCD_E				(1 << LCD_E_PIN)	/**< E-bit in 1st and 2nd higher nibble */
#endif
/*@}*/

// data & control bits for internal use, do not change!

#define CMD_D0				(1 << 0)	/**< bit 0 in lower nibble */
#define CMD_D1				(1 << 1)	/**< bit 1 in lower nibble */
#define CMD_D2				(1 << 2)	/**< bit 2 in lower nibble */
#define CMD_D3				(1 << 3)	/**< bit 3 in lower nibble */
#define CMD_RS				(1 << 4)	/**< RS-bit */
#define CMD_RW				(1 << 5)	/**< RW-bit */


#define LCD_ADDRESS			0	/**< Used for reading the address-counter and busy-flag */
#define LCD_DATA			1	/**< Used for reading data */


//-LCD-COMMANDS------------------------------------------------------------------------------------------------------

#define LCD_CLEAR			0x01	/**< Clear screen */
#define LCD_HOME			0x02	/**< Cursor move to first digit */

#define LCD_ENTRYMODE			0x04			/**< Set entrymode */
	#define LCD_INCREASE		LCD_ENTRYMODE | 0x02	/**<	Set cursor move direction -- Increase */
	#define LCD_DECREASE		LCD_ENTRYMODE | 0x00	/**<	Set cursor move direction -- Decrease */
	#define LCD_DISPLAYSHIFTON	LCD_ENTRYMODE | 0x01	/**<	Display is shifted */
	#define LCD_DISPLAYSHIFTOFF	LCD_ENTRYMODE | 0x00	/**<	Display is not shifted */

#define LCD_DISPLAYMODE			0x08			/**< Set displaymode */
	#define LCD_DISPLAYON		LCD_DISPLAYMODE | 0x04	/**<	Display on */
	#define LCD_DISPLAYOFF		LCD_DISPLAYMODE | 0x00	/**<	Display off */
	#define LCD_CURSORON		LCD_DISPLAYMODE | 0x02	/**<	Cursor on */
	#define LCD_CURSOROFF		LCD_DISPLAYMODE | 0x00	/**<	Cursor off */
	#define LCD_BLINKINGON		LCD_DISPLAYMODE | 0x01	/**<	Blinking on */
	#define LCD_BLINKINGOFF		LCD_DISPLAYMODE | 0x00	/**<	Blinking off */
/*@}*/ 

/** @name SHIFTMODES */
/*@{*/ 
#define LCD_SHIFTMODE			0x10			/**< Set shiftmode */
	#define LCD_DISPLAYSHIFT	LCD_SHIFTMODE | 0x08	/**<	Display shift */
	#define LCD_CURSORMOVE		LCD_SHIFTMODE | 0x00	/**<	Cursor move */
	#define LCD_RIGHT			LCD_SHIFTMODE | 0x04	/**<	Right shift */
	#define LCD_LEFT			LCD_SHIFTMODE | 0x00	/**<	Left shift */
/*@}*/ 

/** @name DISPLAY_CONFIGURATION */
/*@{*/ 
#define LCD_CONFIGURATION		0x20				/**< Set function */
	#define LCD_8BIT		LCD_CONFIGURATION | 0x10	/**<	8 bits interface */
	#define LCD_4BIT		LCD_CONFIGURATION | 0x00	/**<	4 bits interface */
	#define LCD_4LINE						    0x09	/**<	4 line display */
	#define LCD_2LINE		LCD_CONFIGURATION | 0x08	/**<	2 line display */
	#define LCD_1LINE		LCD_CONFIGURATION | 0x00	/**<	1 line display */
	#define LCD_5X10		LCD_CONFIGURATION | 0x04	/**<	5 X 10 dots */
	#define LCD_5X7			LCD_CONFIGURATION | 0x00	/**<	5 X 7 dots */

#define LCD_LIGHT_ON		LCD_LIGHT_N       
#define LCD_LIGHT_OFF		0x00
/*@}*/ 

//---------------------------------------------------------------------

 // global variable fr light control

//static uint8_t lightOn=0;

//-	Write data to i2c
#define I2C_WRITE 0
void twilcd_write_i2c(uint8_t value) {
	i2c_start_wait(LCD_I2C_DEVICE+I2C_WRITE);
	
	i2c_write(value);
	i2c_stop();
}
//---------------------------------------------------------------------

//-	Write nibble to display with pulse of enable bit
// map pinout between PCF8574 and LCD
u8 light_on;
void setandtransfer(uint8_t value) {
    uint8_t data_out=0;
    
    // map data to LCD pinout
    if (value & CMD_D0) data_out |= (1 << LCD_D4_PIN);
    if (value & CMD_D1) data_out |= (1 << LCD_D5_PIN);
    if (value & CMD_D2) data_out |= (1 << LCD_D6_PIN);
    if (value & CMD_D3) data_out |= (1 << LCD_D7_PIN);
    if (value & CMD_RS) data_out |= (1 << LCD_RS_PIN);
    if (value & CMD_RW) data_out |= (1 << LCD_RW_PIN);
    if (light_on) 		data_out |= (1 << LCD_LIGHT_PIN);
	

//put to lowlevel
	twilcd_write_i2c(data_out | (1 << LCD_E_PIN));	//-	Set new data and enable to high
	twilcd_write_i2c(data_out);	            //-	Set enable to low
}

//---------------------------------------------------------------------

//-	Issue a command to the display (use the defined commands above)

void lcd_command(uint8_t command) {
    setandtransfer((command >> 4));
	setandtransfer((command & 0x0F));	
}
//---------------------------------------------------------------------

//-	Display initialization sequence

void lcd_init(void) {
	_delay_ms(25);//15		        //-	Wait for more than 15ms after VDD rises to 4.5V
	setandtransfer(CMD_D1 | CMD_D0);	//-	Set interface to 8-bit
	_delay_ms(5);			    //-	Wait for more than 4.1ms
	setandtransfer(CMD_D1 | CMD_D0);	//-	Set interface to 8-bit
	_delay_ms(1);//0.1		        //-	Wait for more than 100us	
	setandtransfer(CMD_D1 | CMD_D0);	//-	Set interface to 8-bit
	setandtransfer(CMD_D1);		    //-	Set interface to 4-bit

	//- From now on in 4-bit-Mode
	lcd_command(LCD_LINE_MODE | LCD_5X7);
	lcd_command(LCD_DISPLAYON | LCD_CURSORON | LCD_BLINKINGOFF);
	lcd_command(LCD_CLEAR);
    _delay_ms(5);//2
	lcd_command(LCD_INCREASE | LCD_DISPLAYSHIFTOFF);
    _delay_ms(200);//0
}

//---------------------------------------------------------------------

//---------------------------------------------------------------------------
//	tlcd_clear(..) - lscht die Anzeige im LCD
//---------------------------------------------------------------------------
void lcd_clear(uint16_t nothing)
{
	lcd_command(0x01);
	_delay_ms(2);			// warten
}


//xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx




//-	Go to position (line, column)

void lcd_goto(uint8_t line, uint8_t col ) {
	uint8_t lcddata=0;
	
	if( (line > LCD_LINES) ||
	    (col > LCD_COLS) ||
	    ((line == 0) || (col == 0)) ) return 0;
	
    switch (line) {
        case 1: lcddata = LCD_LINE1; break;
        case 2: lcddata = LCD_LINE2; break;
        case 3: lcddata = LCD_LINE3; break;
        case 4: lcddata = LCD_LINE4; break;
    }
    lcddata |= 0x80;
    lcddata += (col-1);
	lcd_command(lcddata);
	//return 1;	
}

//-----------------------------------------------------
//-	Put char to atctual cursor position

void lcd_char(char lcddata) {
	setandtransfer((lcddata >> 4) | CMD_RS);
	setandtransfer((lcddata & 0x0F) | CMD_RS);
}
//---------------------------------------------------
//-	Print string to cursor position

void lcd_write(char *string) {
	while(*string)	{
		lcd_char(*string++);
	}
}

void lcd_light(u8 onoff){
light_on = onoff;
//light_on_timer=120;
//lcd_goto(4,18);
//lcd_int(light_on);

}

void sub_set_Comma(void){;}
 
 char hextablex[18] = "0123456789ABCDEF";


  void  lcd_hex(uint8_t byteX)
  //
  {
  uint8_t x;
char dummy[4]={'x','x','x','\0'};
  x=byteX/16;
 	/*	if(x<10){
		dummy[1]=(x+48);
		}else{
		dummy[1]=(x+55);
		}*/
        
       dummy[1]= hextablex[x];
        
        
	 x=byteX%16;
	/*	if(x<10){
		dummy[2]=(x+48);
		}else{
		dummy[2]=(x+55);
		}*/
       dummy[2]= hextablex[x];
        
	lcd_write(dummy);	
		
  }
//============================================================

  void  lcd_hex16(uint16_t byteX)
  //
  {
 //byteX =65000;
  uint8_t x;
char dummy[6]={'x','0','0','0','0','\0'};
  x=byteX/4096;
 		if(x<10){
		dummy[1]=(x+48);
		}else{
		dummy[1]=(x+55);
		}
  x=(byteX%4096)/256;
 		if(x<10){
		dummy[2]=(x+48);
		}else{
		dummy[2]=(x+55);
		}
		
	 x=(byteX%256)/16;
		if(x<10){
		dummy[3]=(x+48);
		}else{
		dummy[3]=(x+55);
		}
	 x=(byteX%16);
		
 		if(x<10){
		dummy[4]=(x+48);
		}else{
		dummy[4]=(x+55);
		}
	lcd_write(dummy);	
		
  }
  
  
 li4(i16 x){
	uint8_t dum[5]={'+',48,48,48,'\0'};
if (x!=0)	
{
	
if (x<0){x=x*-1;dum[0]='-';}

    if (x>999){
        dum[1]=(x/1000+48);
        x=x%1000;
        }
	
    if (x>99){
        dum[1]=(x/100+48);
        x=x%100;
        }
    if (x>9){
        dum[2]=(x/10+48);
        x=x%10;
        }
         dum[3]=(x+48);
 

}
    lcd_write(dum);
}
 
  
  
  
//============================================================


//---------------------------------------------------------------------
//modus=0 :1000
//modus=1 :1.000
//modus=2 :01
//modus=3 :01_


/*
void twilcd_int(int64_t dat,u8 modus){
if (dat==0)	{lcd_print("0");}
else
{
	uint8_t l=1,j,n=0,p=0;
	uint64_t tl=1;
	uint8_t dum[10];
	
if(dat<0){dum[0]='-';dat=-1*dat;n=1;}

while((dat/tl)>0){
tl=tl*10;l++;
}

tl=tl/10;
for(j=0;j<(l-1);j++){

if(modus==2 && l==2){dum[j+n+p]='0';p++;}

dum[j+n+p]=dat/tl+48;
	
if(modus==1){
if(tl==1000||tl==1000000){p++;dum[j+n+p]='.';}
}


dat=dat%tl;
	tl=tl/10;
	}

if(modus==3){
dum[p+n+l-1]=' ';
p++;
}
	
dum[p+n+l-1]='\0';
lcd_print(dum);
}
}


  void  twilcd_hex(uint8_t byteX)
  //
  {
  uint8_t x;
char dummy[4]={'x','x','x','\0'};
  x=byteX/16;
 		if(x<10){
		dummy[1]=(x+48);
		}else{
		dummy[1]=(x+55);
		}
	 x=byteX%16;
		if(x<10){
		dummy[2]=(x+48);
		}else{
		dummy[2]=(x+55);
		}
	setandtransfer(dummy);	
		
  }


 
void lcd_bitweise(u8 byteX){
u8 i;
 for( i=7;i!=255;i--) {
 twilcd_int((byteX&128)/128,0);
 byteX=byteX<<1;
 }
}

*/

/*

#define lcd_clear(color) twilcd_clear()
#define lcd_time twilcd_time()
#define lcd_write lcd_print
#define lcd_goto twilcd_goto
#define lcd_int(x) twilcd_int(x,0)
#define lcd_int2(x) twilcd_int(x,2)
#define li(x) twilcd_int(x,2)
#define lgi(x,y,z)  twilcd_goto(x,y);twilcd_int(z,2)
#define lw lcd_print
#define lgw(x,y,z)  twilcd_goto(x,y);lcd_print(z)
#define lg lcd_goto
#define lcd_region(a,b,c,d,e,f,g) nothing()
#define selectfont(x) nothing()
#define lcd_hex(x)  twilcd_hex(x)
#define lcd_hex16(x)  twilcd_hex16(x)

*/
/*#define 
#define 
#define 
#define 
*/

#define lcd_region(a,b,c,d,e,f,g) nothing()
#define selectfont(x) nothing()


//---------------------------------------------------
/*
*/



void lcd_int(int32_t dat){

if (dat==0)	{lcd_char(48);}
else
{
u32 l=1000000;
u8 n=0;
u8 y;
if(dat<0){;dat=-1*dat;n=1;
lcd_char('-');
}

while (dat/l==0){
l=l/10;
}

while(l){
y=dat/l;
lcd_char((y)+48);
dat=dat%l;
l=l/10;
}
}
/*


if (dat==0)	{lcd_write("0");}
else
{
	uint8_t l=1,j,n=0;
	uint32_t tl=1;
	uint8_t dum[10];
while((dat/tl)>0){
tl=tl*10;l++;
}

tl=tl/10;
for(j=0;j<(l-1);j++){
	dum[j+n]=dat/tl+48;
	dat=dat%tl;
	tl=tl/10;
	}
dum[n+l-1]='\0';
lcd_write(dum);
}
*/
}


//============================================================
void lcd_int2(uint8_t int1){
	uint8_t dum[3]={48,48,'\0'};
    if (int1>9){
        dum[0]=(int1/10+48);
        int1=int1%10;
        }
        dum[1]=int1+48; 
        
     lcd_write(dum);
}
//============================================================


//---------------------------------------------------
/*
void lcd_time(void){
if(sec==0){
    lcd_goto(1,1);lcd_int2(hou);lcd_write(":");
    lcd_int2(min);lcd_write(":");
    lcd_int2(sec);
}else{
    lcd_goto(1,7);lcd_int2(sec);//lcd_char('#');
}
}
*/
//---------------------------------------------------

